/*
*         OpenPBS (Portable Batch System) v2.3 Software License
*
* Copyright (c) 1999-2000 Veridian Information Solutions, Inc.
* All rights reserved.
*
* ---------------------------------------------------------------------------
* For a license to use or redistribute the OpenPBS software under conditions
* other than those described below, or to purchase support for this software,
* please contact Veridian Systems, PBS Products Department ("Licensor") at:
*
*    www.OpenPBS.org  +1 650 967-4675                  sales@OpenPBS.org
*                        877 902-4PBS (US toll-free)
* ---------------------------------------------------------------------------
*
* This license covers use of the OpenPBS v2.3 software (the "Software") at
* your site or location, and, for certain users, redistribution of the
* Software to other sites and locations.  Use and redistribution of
* OpenPBS v2.3 in source and binary forms, with or without modification,
* are permitted provided that all of the following conditions are met.
* After December 31, 2001, only conditions 3-6 must be met:
*
* 1. Commercial and/or non-commercial use of the Software is permitted
*    provided a current software registration is on file at www.OpenPBS.org.
*    If use of this software contributes to a publication, product, or
*    service, proper attribution must be given; see www.OpenPBS.org/credit.html
*
* 2. Redistribution in any form is only permitted for non-commercial,
*    non-profit purposes.  There can be no charge for the Software or any
*    software incorporating the Software.  Further, there can be no
*    expectation of revenue generated as a consequence of redistributing
*    the Software.
*
* 3. Any Redistribution of source code must retain the above copyright notice
*    and the acknowledgment contained in paragraph 6, this list of conditions
*    and the disclaimer contained in paragraph 7.
*
* 4. Any Redistribution in binary form must reproduce the above copyright
*    notice and the acknowledgment contained in paragraph 6, this list of
*    conditions and the disclaimer contained in paragraph 7 in the
*    documentation and/or other materials provided with the distribution.
*
* 5. Redistributions in any form must be accompanied by information on how to
*    obtain complete source code for the OpenPBS software and any
*    modifications and/or additions to the OpenPBS software.  The source code
*    must either be included in the distribution or be available for no more
*    than the cost of distribution plus a nominal fee, and all modifications
*    and additions to the Software must be freely redistributable by any party
*    (including Licensor) without restriction.
*
* 6. All advertising materials mentioning features or use of the Software must
*    display the following acknowledgment:
*
*     "This product includes software developed by NASA Ames Research Center,
*     Lawrence Livermore National Laboratory, and Veridian Information
*     Solutions, Inc.
*     Visit www.OpenPBS.org for OpenPBS software support,
*     products, and information."
*
* 7. DISCLAIMER OF WARRANTY
*
* THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. ANY EXPRESS
* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT
* ARE EXPRESSLY DISCLAIMED.
*
* IN NO EVENT SHALL VERIDIAN CORPORATION, ITS AFFILIATED COMPANIES, OR THE
* U.S. GOVERNMENT OR ANY OF ITS AGENCIES BE LIABLE FOR ANY DIRECT OR INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
* OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* This license will be governed by the laws of the Commonwealth of Virginia,
* without reference to its choice of law rules.
*/

#include <pbs_config.h>   /* the master config generated by configure */

#include <sys/types.h>
#include <stdlib.h>
#include "libpbs.h"
#include "list_link.h"
#include "attribute.h"
#include "credential.h"
#include "batch_request.h"
#include "pbs_error.h"
#include "dis.h"

/*
 * decode_DIS_rescl() - decode a resource request
 *
 * Used for resource query, resource reserver, resource free.
 *
 * The batch_request structure must already exist (be allocated by the
 * caller.   It is assumed that the header fields (protocol type,
 * protocol version, request type, and user name) have been decoded.
 *
 * Data items are: signed int resource handle
 *   unsigned int count of resource queries
 *  followed by that number of:
 *   string  resource list
 */

int decode_DIS_Rescl(

  int  sock,
  struct batch_request *preq)

  {
  int    ct;
  int    i;
  char **ppc;
  int    rc;

  /* first, the resource handle (even if not used in request) */

  preq->rq_ind.rq_rescq.rq_rhandle = disrsi(sock, &rc);

  if (rc)
    {
    return(rc);
    }

  /* next need to know how many query strings */

  ct = disrui(sock, &rc);

  if (rc)
    {
    return(rc);
    }

  preq->rq_ind.rq_rescq.rq_num = ct;

  if (ct)
    {
    if ((ppc = (char **)malloc(ct * sizeof(char *))) == NULL)
      {
      return(PBSE_RMSYSTEM);
      }

    for (i = 0;i < ct;i++)
      *(ppc + i) = NULL;

    preq->rq_ind.rq_rescq.rq_list = ppc;

    for (i = 0;i < ct;i++)
      {
      *(ppc + i) = disrst(sock, &rc);

      if (rc)
        break;
      }
    }

  return(rc);
  }  /* END decode_DIS_Rescl() */


int tcp_decode_DIS_Rescl(

  int  sock,
  struct batch_request *preq)

  {
  int    ct;
  int    i;
  char **ppc;
  int    rc;

  /* first, the resource handle (even if not used in request) */

  preq->rq_ind.rq_rescq.rq_rhandle = tcp_disrsi(sock, &rc);

  if (rc)
    {
    return(rc);
    }

  /* next need to know how many query strings */

  ct = tcp_disrui(sock, &rc);

  if (rc)
    {
    return(rc);
    }

  preq->rq_ind.rq_rescq.rq_num = ct;

  if (ct)
    {
    if ((ppc = (char **)malloc(ct * sizeof(char *))) == NULL)
      {
      return(PBSE_RMSYSTEM);
      }

    for (i = 0;i < ct;i++)
      *(ppc + i) = NULL;

    preq->rq_ind.rq_rescq.rq_list = ppc;

    for (i = 0;i < ct;i++)
      {
      *(ppc + i) = tcp_disrst(sock, &rc);

      if (rc)
        break;
      }
    }

  return(rc);
  }  /* END tcp_decode_DIS_Rescl() */


